// RestartViaManager.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

int restartApp(const bool fQuiet, const wchar_t *szExePath);
int restartApp(const bool fQuiet, const wchar_t *szExePath, const DWORD dwRMSessionHandle);


int _tmain(int argc, _TCHAR* argv[])
{
	bool fQuiet = false;
	const wchar_t *szExePath = nullptr;

	if (argc == 2)
	{
		szExePath = argv[1];
	}
	else if (argc == 3)
	{
		if (_wcsicmp(argv[1], L"quiet") == 0)
		{
			fQuiet = true;
			szExePath = argv[2];
		}
		else
		if (_wcsicmp(argv[2], L"quiet") == 0)
		{
			fQuiet = true;
			szExePath = argv[1];
		}
	}

	if (szExePath == nullptr)
	{
		wprintf(L"Restarts a process using the Windows Restart Manager API.\n");
		wprintf(L"The process must be running, and must have opted in to being restartable via the Restart Manager.\n");
		wprintf(L"Usage:   RestartViaManager.exe \"<full path to process exe>\" [QUIET]\n");
		wprintf(L"Example: RestartViaManager.exe \"C:\\windows\\explorer.exe\"\n");
		wprintf(L"The optional QUIET argument will mean nothing is printed if the command is successful.\n");
		return -1;
	}

	if (::CoInitialize(0) != S_OK)
	{
		wprintf(L"RestartViaManager: Failed to initialize COM.\r\n");
		return -1;
	}

	int iResult = restartApp(fQuiet, szExePath);

	::CoUninitialize();

	return iResult;
}

int restartApp(const bool fQuiet, const wchar_t *szExePath)
{
	DWORD dwAttr = ::GetFileAttributes(szExePath);
	if (dwAttr == INVALID_FILE_ATTRIBUTES)
	{
		wprintf(L"RestartViaManager: \"%s\" not found.\n", szExePath);
		return -1;
	}

	if (dwAttr & FILE_ATTRIBUTE_DIRECTORY)
	{
		wprintf(L"RestartViaManager: \"%s\" is a directory.\n", szExePath);
		return -1;
	}

	DWORD dwRMSessionHandle = 0;
	wchar_t szRMSessionKey[ max( CCH_RM_SESSION_KEY+1, 1024 ) ] = {0};

	DWORD dwSessionRes = ::RmStartSession(&dwRMSessionHandle, 0, szRMSessionKey);

	if (dwSessionRes != ERROR_SUCCESS)
	{
		wprintf(L"RestartViaManager: Failed to start Restart Manager session. (Error %ld)\n", dwSessionRes);
		return -1;
	}

	int iResult = restartApp(fQuiet, szExePath, dwRMSessionHandle);

	dwSessionRes = ::RmEndSession(dwRMSessionHandle);
	
	if (dwSessionRes != ERROR_SUCCESS) // && iResult == 0)
	{
		wprintf(L"RestartViaManager: Failed to end Restart Manager session. (Error %ld)\n", dwSessionRes);
		return -1;
	}

	return iResult;
}

int restartApp(const bool fQuiet, const wchar_t *szExePath, const DWORD dwRMSessionHandle)
{
	DWORD dwRes = ::RmRegisterResources(dwRMSessionHandle, 1, &szExePath, 0, NULL, 0, NULL);

	if (dwRes != ERROR_SUCCESS)
	{
		wprintf(L"RestartViaManager: Failed to add \"%s\" to Restart Manager's resource list. (Error %ld)\n", szExePath, dwRes);
		return -1;
	}

	dwRes = ::RmShutdown(dwRMSessionHandle, 0, NULL);

	if (dwRes != ERROR_SUCCESS)
	{
		wprintf(L"RestartViaManager: Failed to shutdown \"%s\" via Restart Manager. (Error %ld)\n", szExePath, dwRes);
		return -1;
	}

	dwRes = ::RmRestart(dwRMSessionHandle, 0, NULL);

	if (dwRes != ERROR_SUCCESS)
	{
		wprintf(L"RestartViaManager: Failed to restart \"%s\" via Restart Manager. (Error %ld)\n", szExePath, dwRes);
		return -1;
	}

	if (!fQuiet)
	{
		wprintf(L"RestartViaManager: Shutdown and restart requests succeeded for \"%s\".\n", szExePath);
	}

	return 0;
}
